/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.keywords;

import java.util.Collection;

import fr.loof.fonzie.parsers.Constraint;

public class In
    extends AbstractConstraint
    implements Constraint
{
    public In()
    {
        super( "in" );
    }

    /**
     * {@inheritDoc}
     *
     * @see fr.loof.fonzie.parsers.KeyWord#checkParameterType(java.lang.Class, java.lang.String, java.lang.Class,
     *      java.lang.Class)
     */
    @Override
    protected void checkParameterType( Class entity, String property, Class propertyType, Class arg )
    {
        if ( arg.isArray() )
        {
            Class type = arg.getComponentType();
            if ( type.isAssignableFrom( propertyType ) )
            {
                return;
            }

            throw new IllegalStateException( "Constraint on " + entity.getSimpleName() + "." + property + " ("
                + propertyType.getSimpleName() + ") is not compatible with parameter type " + type + "[]" );
        }
        if ( arg.isAssignableFrom( Collection.class ) )
        {
            // No way to check collection type :'(
            return;
        }
        throw new IllegalStateException( "IN constraint requires either an Array or Collection parameter" );
    }
}